def validate_transaction_exists(*, amount, error, recipient, txs):
    """
    Check for the existence of a Tx
    """

    tx = next((tx for tx in txs if tx.get('amount') >= amount and tx.get('recipient') == recipient), None)

    if not tx:
        raise error({
            'error_message': 'Tx not found',
            'expected_amount': amount,
            'expected_recipient': recipient
        })



from rest_framework import serializers

from thenewboston.constants.network import MAX_POINT_VALUE, MIN_POINT_VALUE, VERIFY_KEY_LENGTH
from thenewboston.utils.serializers import validate_keys


class NetworkTransactionSerializer(serializers.Serializer):
    amount = serializers.IntegerField(max_value=MAX_POINT_VALUE, min_value=MIN_POINT_VALUE)
    recipient = serializers.CharField(max_length=VERIFY_KEY_LENGTH, min_length=VERIFY_KEY_LENGTH)

    def create(self, validated_data):
        pass

    def update(self, instance, validated_data):
        pass

    def validate(self, data):
        """
        Check that there are no additional keys included in the data
        """

        validate_keys(self, data)

        return data

    @staticmethod
    def validate_amount(amount):
        """
        Check that amount is not 0
        """

        if amount == 0:
            raise serializers.ValidationError('Tx amount can not be 0 (Tx should be excluded)')

        return amount



from rest_framework import serializers

from thenewboston.blocks.signatures import verify_signature
from thenewboston.constants.network import SIGNATURE_LENGTH, VERIFY_KEY_LENGTH
from thenewboston.serializers.message import MessageSerializer
from thenewboston.utils.serializers import validate_keys
from thenewboston.utils.tools import sort_and_encode


class NetworkBlockSerializer(serializers.Serializer):
    account_number = serializers.CharField(max_length=VERIFY_KEY_LENGTH, min_length=VERIFY_KEY_LENGTH)
    message = MessageSerializer()
    signature = serializers.CharField(max_length=SIGNATURE_LENGTH, min_length=SIGNATURE_LENGTH)

    def create(self, validated_data):
        pass

    def update(self, instance, validated_data):
        pass

    def validate(self, data):
        """
        Validate signature
        Validate Tx recipients are unique
        Validate account_number (the sender) is not included as a Tx recipient
        """

        account_number = data['account_number']
        message = data['message']
        txs = message['txs']
        signature = data['signature']

        verify_signature(
            message=sort_and_encode(message),
            signature=signature,
            verify_key=account_number
        )

        recipient_list = [tx['recipient'] for tx in txs]
        recipient_set = set(recipient_list)

        if len(recipient_list) != len(recipient_set):
            raise serializers.ValidationError('Tx recipients must be unique')

        if account_number in recipient_set:
            raise serializers.ValidationError('Block account_number not allowed as Tx recipient')

        validate_keys(self, data)

        return data



import json
from hashlib import sha3_256 as sha3


def get_file_hash(file):
    """
    Return hash value of file
    """

    h = sha3()

    with open(file, 'rb') as file:
        chunk = 0

        while chunk != b'':
            chunk = file.read(1024)
            h.update(chunk)

    return h.hexdigest()


def read_json(file):
    """
    Read JSON file
    """

    try:
        with open(file, 'r') as f:
            data = json.load(f)
    except FileNotFoundError:
        data = None
    return data


def write_json(file, data):
    """
    Write JSON file
    """

    with open(file, 'w') as f:
        json.dump(data, f, indent=2)
